import asyncio, random

from hangupsbot.utils import strip_quotes, text_to_segments
from hangupsbot.commands import command


@asyncio.coroutine
def easteregg_combo(client, conv_id, easteregg, eggcount=1, period=0.5):
    """Send easter egg combo (ponies, pitchforks, bikeshed, shydino)"""
    for i in range(eggcount):
        yield from client.sendeasteregg(conv_id, easteregg)
        if eggcount > 1:
            yield from asyncio.sleep(period + random.uniform(-0.1, 0.1))


@command.register(admin=True)
def easteregg(bot, event, easteregg, eggcount=1, period=0.5, conv_name='', *args):
    """Annoy people with easter egg combo in current (or specified) conversation!
       Usage: /bot easteregg easter_egg_type [count] [period] [conv_name]
       Supported easter eggs: ponies, pitchforks, bikeshed, shydino"""
    conv_name = strip_quotes(conv_name)
    convs = [event.conv] if not conv_name or conv_name == '.' else bot.find_conversations(conv_name)
    for c in convs:
        asyncio.async(
            easteregg_combo(bot._client, c.id_, easteregg, int(eggcount), float(period))
        ).add_done_callback(lambda future: future.result())


@command.register
def spoof(bot, event, *args):
    """Spoof IngressBot on specified GPS coordinates
       Usage: /bot spoof latitude,longitude [hack|fire|deploy|mod] [level] [count]"""
    link = 'https://plus.google.com/u/0/{}/about'.format(event.user.id_.chat_id)
    text = _(
        '**!!! WARNING !!!**\n'
        'Agent {} ({}) has been reported to Niantic for attempted spoofing!'
    ).format(event.user.full_name, link)
    yield from event.conv.send_message(text_to_segments(text))



#!/usr/bin/env python

# Install modern gettext class-based API in Python's builtins namespace first
import os, io, gettext
localedir = os.path.join(os.path.dirname(__file__), 'locale')
gettext.install('hangupsbot', localedir=localedir)

# For argparse localization to work, we need also to setup old GNU gettext API
gettext.bindtextdomain('hangupsbot', localedir=localedir)
gettext.textdomain('hangupsbot')

import sys, argparse, logging, shutil, asyncio, time, signal

import appdirs
import hangups
from hangups import http_utils
from hangups.conversation import Conversation
from hangups.ui.utils import get_conv_name

import hangupsbot.config
from hangupsbot.version import __version__
from hangupsbot.utils import text_to_segments
from hangupsbot.handlers import handler


LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'


class HangupsBot:
    """Hangouts bot listening on all conversations"""
    def __init__(self, cookies_path, config_path, max_retries=5):
        self._client = None
        self._cookies_path = cookies_path
        self._max_retries = max_retries

        # These are populated by on_connect when it's called.
        self._conv_list = None        # hangups.ConversationList
        self._user_list = None        # hangups.UserList

        # Load config file
        self.config = hangupsbot.config.Config(config_path)

        # Handle signals on Unix
        # (add_signal_handler is not implemented on Windows)
        try:
            loop = asyncio.get_event_loop()
            for signum in (signal.SIGINT, signal.SIGTERM):
                loop.add_signal_handler(signum, lambda: self.stop())
        except NotImplementedError:
            pass

    def login(self, cookies_path):
        """Login to Google account"""
        # Authenticate Google user and save auth cookies
        # (or load already saved cookies)
        try:
            cookies = hangups.auth.get_auth_stdin(cookies_path)
            return cookies
        except hangups.GoogleAuthError as e:
            print(_('Login failed ({})').format(e))
            return False

    def run(self):
        """Connect to Hangouts and run bot"""
        cookies = self.login(self._cookies_path)
        if cookies:
            for retry in range(self._max_retries):
                try:
                    # Create Hangups client
                    self._client = hangups.Client(cookies)
                    self._client.on_connect.add_observer(self._on_connect)
                    self._client.on_disconnect.add_observer(self._on_disconnect)

                    # Start asyncio event loop and connect to Hangouts
                    # If we are forcefully disconnected, try connecting again
                    loop = asyncio.get_event_loop()
                    loop.run_until_complete(self._client.connect())
                    sys.exit(0)
                except Exception as e:
                    print(_('Client unexpectedly disconnected:\n{}').format(e))
                    print(_('Waiting {} seconds...').format(5 + retry * 5))
                    time.sleep(5 + retry * 5)
                    print(_('Trying to connect again (try {} of {})...').format(retry + 1, self._max_retries))
            print(_('Maximum number of retries reached! Exiting...'))
        sys.exit(1)

    def stop(self):
        """Disconnect from Hangouts"""
        asyncio.async(
            self._client.disconnect()
        ).add_done_callback(lambda future: future.result())

    def send_message(self, conversation, text):
        """Send simple chat message"""
        self.send_message_segments(conversation, text_to_segments(text))

    def send_message_segments(self, conversation, segments):
        """Send chat message segments"""
        # Ignore if the user hasn't typed a message.
        if len(segments) == 0:
            return
        # XXX: Exception handling here is still a bit broken. Uncaught
        # exceptions in _on_message_sent will only be logged.
        asyncio.async(
            conversation.send_message(segments)
        ).add_done_callback(self._on_message_sent)

    @asyncio.coroutine
    def upload_images(self, links):
        """Download images and upload them to Google+"""
        image_id_list = []
        for link in links:
            # Download image
            try:
                res = yield from http_utils.fetch('get', link)
            except hangups.NetworkError as e:
                print('Failed to download image: {}'.format(e))
                continue

            # Upload image and get image_id
            try:
                image_id = yield from self._client.upload_image(io.BytesIO(res.body),
                                                                filename=os.path.basename(link))
                image_id_list.append(image_id)
            except hangups.NetworkError as e:
                print('Failed to upload image: {}'.format(e))
                continue
        return image_id_list

    def list_conversations(self):
        """List all active conversations"""
        convs = sorted(self._conv_list.get_all(),
                       reverse=True, key=lambda c: c.last_modified)
        return convs

    def find_conversations(self, conv_name):
        """Find conversations by name or ID in list of all active conversations"""
        conv_name = conv_name.strip()
        conv_name_lower = conv_name.lower()
        if conv_name_lower.startswith("id:"):
            return [self._conv_list.get(conv_name[3:])]

        convs = [c for c in self.list_conversations()
                 if conv_name_lower in get_conv_name(c, truncate=True).lower()]
        return convs

    def list_users(self, conv=None):
        """List all known users or all users in conversation"""
        def full_name_sort(user):
            split_name = user.full_name.split()
            return (split_name[-1], split_name[0])
        users = conv.users if isinstance(conv, Conversation) else self._user_list.get_all()
        return sorted(users, key=full_name_sort)

    def find_users(self, user_name, conv=None):
        """Find users by name or ID in list of all known users or in conversation"""
        user_name = user_name.strip()
        user_name_lower = user_name.lower()
        if user_name_lower.startswith("id:"):
            return [self._user_list.get(user_name[3:])]

        users = [u for u in self.list_users(conv=conv)
                 if user_name_lower in u.full_name.lower()]
        return users

    def get_config_suboption(self, conv_id, option):
        """Get config suboption for conversation (or global option if not defined)"""
        try:
            suboption = self.config['conversations'][conv_id][option]
        except KeyError:
            try:
                suboption = self.config[option]
            except KeyError:
                suboption = None
        return suboption

    def _on_message_sent(self, future):
        """Handle showing an error if a message fails to send"""
        try:
            future.result()
        except hangups.NetworkError:
            print(_('Failed to send message!'))

    @asyncio.coroutine
    def _on_connect(self, initial_data):
        """Handle connecting for the first time"""
        print(_('Connected!'))
        self._user_list = yield from hangups.build_user_list(
            self._client,
            initial_data
        )
        self._conv_list = hangups.ConversationList(
            self._client,
            initial_data.conversation_states,
            self._user_list,
            initial_data.sync_timestamp
        )
        self._conv_list.on_event.add_observer(self._on_event)

        print(_('Conversations:'))
        for c in self.list_conversations():
            print('  {} ({})'.format(get_conv_name(c, truncate=True), c.id_))
        print()

    @asyncio.coroutine
    def _on_event(self, conv_event):
        """Handle conversation events"""
        yield from handler.handle(self, conv_event)

    @asyncio.coroutine
    def _on_disconnect(self):
        """Handle disconnecting"""
        print(_('Connection lost!'))


def main():
    """Main entry point"""
    # Build default paths for files.
    dirs = appdirs.AppDirs('hangupsbot', 'hangupsbot')
    default_log_path = os.path.join(dirs.user_data_dir, 'hangupsbot.log')
    default_cookies_path = os.path.join(dirs.user_data_dir, 'cookies.json')
    default_config_path = os.path.join(dirs.user_data_dir, 'config.json')

    # Configure argument parser
    parser = argparse.ArgumentParser(prog='hangupsbot',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--debug', action='store_true',
                        help=_('log detailed debugging messages'))
    parser.add_argument('--log', default=default_log_path,
                        help=_('log file path'))
    parser.add_argument('--cookies', default=default_cookies_path,
                        help=_('cookie storage path'))
    parser.add_argument('--config', default=default_config_path,
                        help=_('config storage path'))
    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(__version__),
                        help=_('show program\'s version number and exit'))
    args = parser.parse_args()

    # Create all necessary directories.
    for path in [args.log, args.cookies, args.config]:
        directory = os.path.dirname(path)
        if directory and not os.path.isdir(directory):
            try:
                os.makedirs(directory)
            except OSError as e:
                sys.exit(_('Failed to create directory: {}').format(e))

    # If there is no config file in user data directory, copy default one there
    if not os.path.isfile(args.config):
        try:
            shutil.copy(os.path.abspath(os.path.join(os.path.dirname(__file__), 'config.json')),
                        args.config)
        except (OSError, IOError) as e:
            sys.exit(_('Failed to copy default config file: {}').format(e))

    # Configure logging
    log_level = logging.DEBUG if args.debug else logging.WARNING
    logging.basicConfig(filename=args.log, level=log_level, format=LOG_FORMAT)
    # asyncio's debugging logs are VERY noisy, so adjust the log level
    logging.getLogger('asyncio').setLevel(logging.WARNING)

    # Start Hangups bot
    bot = HangupsBot(args.cookies, args.config)
    bot.run()


if __name__ == '__main__':
    main()



import collections, functools, json


class Config(collections.MutableMapping):
    """Configuration JSON storage class"""
    def __init__(self, filename, default=None):
        self.filename = filename
        self.default = None
        self.config = {}
        self.changed = False
        self.load()

    def load(self):
        """Load config from file"""
        try:
            self.config = json.load(open(self.filename))
        except IOError:
            self.config = {}
        self.changed = False

    def loads(self, json_str):
        """Load config from JSON string"""
        self.config = json.loads(json_str)
        self.changed = True

    def save(self):
        """Save config to file (only if config has changed)"""
        if self.changed:
            with open(self.filename, 'w') as f:
                json.dump(self.config, f, indent=2, sort_keys=True)
                self.changed = False

    def get_by_path(self, keys_list):
        """Get item from config by path (list of keys)"""
        return functools.reduce(lambda d, k: d[k], keys_list, self)

    def set_by_path(self, keys_list, value):
        """Set item in config by path (list of keys)"""
        self.get_by_path(keys_list[:-1])[keys_list[-1]] = value

    def __getitem__(self, key):
        try:
            return self.config[key]
        except KeyError:
            return self.default

    def __setitem__(self, key, value):
        self.config[key] = value
        self.changed = True

    def __delitem__(self, key):
        del self.config[key]
        self.changed = True

    def __iter__(self):
        return iter(self.config)

    def __len__(self):
        return len(self.config)



import re

import hangups

from hangupsbot.utils import word_in_text, text_to_segments
from hangupsbot.handlers import handler


def find_keyword(kw, text):
    """Return True if keyword is in text"""
    if kw == "*":
        return True
    elif kw.lower().startswith("regex:") and re.search(kw[6:], text, re.DOTALL | re.IGNORECASE):
        return True
    elif word_in_text(kw, text):
        return True
    else:
        return False


@handler.register(priority=7, event=hangups.ChatMessageEvent)
def handle_autoreply(bot, event):
    """Handle autoreplies to keywords in messages"""
    # Test if message is not empty
    if not event.text:
        return

    # Test if autoreplies are enabled
    if not bot.get_config_suboption(event.conv_id, 'autoreplies_enabled'):
        return

    # Test if there are actually any autoreplies
    autoreplies_list = bot.get_config_suboption(event.conv_id, 'autoreplies')
    if not autoreplies_list:
        return

    for kwds, sentence in autoreplies_list:
        for kw in kwds:
            if find_keyword(kw, event.text):
                yield from event.conv.send_message(text_to_segments(sentence))
                break



import hangups

from hangupsbot.utils import text_to_segments
from hangupsbot.handlers import handler


@handler.register(priority=7, event=hangups.ChatMessageEvent)
def handle_forward(bot, event):
    """Handle message forwarding"""
    # Test if message is not empty
    if not event.text:
        return

    # Test if message forwarding is enabled
    if not bot.get_config_suboption(event.conv_id, 'forwarding_enabled'):
        return

    # Test if there are actually any forwarding destinations
    forward_to_list = bot.get_config_suboption(event.conv_id, 'forward_to')
    if not forward_to_list:
        return

    # Prepare attachments
    image_id_list = yield from bot.upload_images(event.conv_event.attachments)

    # Forward message to all destinations
    for dst in forward_to_list:
        try:
            conv = bot._conv_list.get(dst)
        except KeyError:
            continue

        # Prepend forwarded message with name of sender
        link = 'https://plus.google.com/u/0/{}/about'.format(event.user_id.chat_id)
        segments = text_to_segments('**[{}]({}):** '.format(event.user.full_name, link))

        # Copy original message segments
        segments.extend(event.conv_event.segments)

        # Send text message first (without attachments)
        yield from conv.send_message(segments)

        # If there are attachments, send them separately
        for image_id in image_id_list:
            yield from conv.send_message([], image_id=image_id)



import os, glob, asyncio


class CommandDispatcher:
    """Register commands and run them"""
    def __init__(self):
        self.commands = {}
        self.commands_admin = []
        self.unknown_command = None

    def get_admin_commands(self, bot, conv_id):
        """Get list of admin-only commands (set by plugins or in config.json)"""
        commands_admin = bot.get_config_suboption(conv_id, 'commands_admin') or []
        return list(set(commands_admin + self.commands_admin))

    @asyncio.coroutine
    def run(self, bot, event, *args, **kwds):
        """Run command"""
        try:
            func = self.commands[args[0]]
        except KeyError:
            if self.unknown_command:
                func = self.unknown_command
            else:
                raise

        args = list(args[1:])

        try:
            yield from func(bot, event, *args, **kwds)
        except Exception as e:
            print(e)

    def register(self, *args, admin=False):
        """Decorator for registering command"""
        def wrapper(func):
            # Automatically wrap command function in coroutine
            func = asyncio.coroutine(func)
            self.commands[func.__name__] = func
            if admin:
                self.commands_admin.append(func.__name__)
            return func

        # If there is one (and only one) positional argument and this argument is callable,
        # assume it is the decorator (without any optional keyword arguments)
        if len(args) == 1 and callable(args[0]):
            return wrapper(args[0])
        else:
            return wrapper

    def register_unknown(self, func):
        """Decorator for registering unknown command"""
        # Automatically wrap command function in coroutine
        func = asyncio.coroutine(func)
        self.unknown_command = func
        return func


# Create CommandDispatcher singleton
command = CommandDispatcher()

# Build list of commands
_plugins = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))
__all__ = [os.path.splitext(os.path.basename(f))[0] for f in _plugins
           if os.path.isfile(f) and not os.path.basename(f).startswith("_")]

# Load all commands
from hangupsbot.commands import *



import json

from hangupsbot.utils import text_to_segments
from hangupsbot.commands import command


@command.register(admin=True)
def config(bot, event, cmd=None, *args):
    """Show or change bot configuration
       Usage: /bot config [get|set] [key] [subkey] [...] [value]"""

    if cmd == 'get' or cmd is None:
        config_args = list(args)
        value = bot.config.get_by_path(config_args) if config_args else dict(bot.config)
    elif cmd == 'set':
        config_args = list(args[:-1])
        if len(args) >= 2:
            bot.config.set_by_path(config_args, json.loads(args[-1]))
            bot.config.save()
            value = bot.config.get_by_path(config_args)
        else:
            yield from command.unknown_command(bot, event)
            return
    else:
        yield from command.unknown_command(bot, event)
        return

    if value is None:
        value = _('Key not found!')

    config_path = ' '.join(k for k in ['config'] + config_args)
    text = (
        '**{}:**\n'
        '{}'
    ).format(config_path, json.dumps(value, indent=2, sort_keys=True))
    yield from event.conv.send_message(text_to_segments(text))


@command.register(admin=True)
def config_reload(bot, event, *args):
    """Reload bot configuration from file"""
    bot.config.load()
    yield from event.conv.send_message(text_to_segments(_('Configuration reloaded')))



import re

from reparser import Parser, Token, MatchGroup
from hangups import ChatMessageSegment, SegmentType


# Common regex patterns
boundary_chars = r'\s`!()\[\]{{}};:\'".,<>?«»“”‘’'
b_left = r'(?:(?<=[' + boundary_chars + r'])|(?<=^))' # Lookbehind
b_right = r'(?:(?=[' + boundary_chars + r'])|(?=$))' # Lookahead

# Regex patterns used by token definitions
markdown = b_left + r'(?P<start>{tag})(?P<text>\S.+?\S)(?P<end>{tag})' + b_right
markdown_link = r'(?P<start>\[)(?P<text>.+?)\]\((?P<url>.+?)(?P<end>\))'
html = r'(?i)(?P<start><{tag}>)(?P<text>.+?)(?P<end></{tag}>)'
html_link = r'(?i)(?P<start><a href=[\'"](?P<url>.+?)[\'"]>)(?P<text>.+?)(?P<end></a>)'
html_img = r'(?i)(?P<text><img src=[\'"](?P<url>.+?)[\'"](\s*/)?>)'
html_newline = r'(?i)(?P<text><br(\s*/)?>)'
newline = r'(?P<text>\n|\r\n)'

# Based on URL regex pattern by John Gruber (http://gist.github.com/gruber/249502)
auto_link = (r'(?i)\b(?P<text>'
             r'(?:[a-z][\w-]+:/{1,3}|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)'
             r'(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+'
             r'(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?«»“”‘’]))')

url_proto_re = re.compile(r'(?i)^[a-z][\w-]+:/{1,3}')
url_complete = lambda u: u if url_proto_re.search(u) else 'http://' + u

class Tokens:
    """Groups of tokens to be used by ChatMessageParser"""
    basic = [
        Token(auto_link, link_target=MatchGroup('text', func=url_complete)),
        Token(newline, text='\n', segment_type=SegmentType.LINE_BREAK)
    ]

    markdown = [
        Token(markdown.format(tag=r'\*\*\*'), is_bold=True, is_italic=True),
        Token(markdown.format(tag=r'___'), is_bold=True, is_italic=True),
        Token(markdown.format(tag=r'\*\*'), is_bold=True),
        Token(markdown.format(tag=r'__'), is_bold=True),
        Token(markdown.format(tag=r'\*'), is_italic=True),
        Token(markdown.format(tag=r'_'), is_italic=True),
        Token(markdown.format(tag=r'~~'), is_strikethrough=True),
        Token(markdown.format(tag=r'=='), is_underline=True),
        Token(markdown_link, link_target=MatchGroup('url', func=url_complete))
    ]

    html = [
        Token(html.format(tag=r'b'), is_bold=True),
        Token(html.format(tag=r'strong'), is_bold=True),
        Token(html.format(tag=r'i'), is_italic=True),
        Token(html.format(tag=r'em'), is_italic=True),
        Token(html.format(tag=r's'), is_strikethrough=True),
        Token(html.format(tag=r'strike'), is_strikethrough=True),
        Token(html.format(tag=r'del'), is_strikethrough=True),
        Token(html.format(tag=r'u'), is_underline=True),
        Token(html.format(tag=r'ins'), is_underline=True),
        Token(html.format(tag=r'mark'), is_underline=True),
        Token(html_link, link_target=MatchGroup('url', func=url_complete)),
        Token(html_img, text=MatchGroup('url', func=url_complete),
              link_target=MatchGroup('url', func=url_complete)),
        Token(html_newline, text='\n', segment_type=SegmentType.LINE_BREAK)
    ]


class ChatMessageParser(Parser):
    """Chat message parser"""
    def __init__(self, tokens=Tokens.markdown + Tokens.html + Tokens.basic):
        super().__init__(tokens)

    def preprocess(self, text):
        """Preprocess text before parsing"""
        # Replace two consecutive spaces with space and non-breakable space
        # (this is how original Hangouts client does it to preserve multiple spaces)
        return text.replace('  ', ' \xa0')

    def parse(self, text):
        """Parse text to obtain list of ChatMessageSegments"""
        segment_list = super().parse(text)
        return [ChatMessageSegment(segment.text, **segment.params) for segment in segment_list]



#!/usr/bin/env python

from distutils.core import setup
from hangupsbot.version import __version__

setup(name='HangupsBot',
      version=__version__,
      description='Bot for Google Hangouts',
      author='Michal Krenek (Mikos)',
      author_email='m.krenek@gmail.com',
      url='https://github.com/xmikos/hangupsbot',
      license="GNU GPLv3",
      packages=['hangupsbot', 'hangupsbot.handlers', 'hangupsbot.commands'],
      package_data={'hangupsbot': ['config.json',
                                   'locale/*/*/*.mo']},
      scripts=['scripts/hangupsbot'],
      requires=['hangups', 'ReParser', 'appdirs', 'asyncio'],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Topic :: Communications :: Chat'
      ])



import shlex

import hangups

from hangupsbot.utils import text_to_segments
from hangupsbot.handlers import handler, StopEventHandling
from hangupsbot.commands import command


bot_command = "/bot"


@handler.register(priority=5, event=hangups.ChatMessageEvent)
def handle_command(bot, event):
    """Handle command messages"""
    # Test if message starts with bot command keyword
    if not event.text or event.text.split()[0].lower() != bot_command:
        return

    # Test if command handling is enabled
    if not bot.get_config_suboption(event.conv_id, 'commands_enabled'):
        raise StopEventHandling

    # Parse message
    line_args = shlex.split(event.text, posix=False)

    # Test if command length is sufficient
    if len(line_args) < 2:
        yield from event.conv.send_message(
            text_to_segments(_('{}: How may I serve you?').format(event.user.full_name))
        )
        raise StopEventHandling

    # Test if user has permissions for running command
    commands_admin_list = command.get_admin_commands(bot, event.conv_id)
    if commands_admin_list and line_args[1].lower() in commands_admin_list:
        admins_list = bot.get_config_suboption(event.conv_id, 'admins')
        if event.user_id.chat_id not in admins_list:
            yield from event.conv.send_message(
                text_to_segments(_('{}: I\'m sorry, Dave. I\'m afraid I can\'t do that.').format(event.user.full_name))
            )
            raise StopEventHandling

    # Run command
    yield from command.run(bot, event, *line_args[1:])

    # Prevent other handlers from processing event
    raise StopEventHandling



import os, glob, logging, itertools, asyncio

import hangups
from hangups.ui.utils import get_conv_name


class StopEventHandling(Exception):
    """Raise to stop handling of current event by other handlers"""
    pass


class ConversationEvent:
    """Cenversation event wrapper"""
    def __init__(self, bot, conv_event):
        self.conv_event = conv_event
        self.conv_id = conv_event.conversation_id
        self.conv = bot._conv_list.get(self.conv_id)
        self.user_id = conv_event.user_id
        self.user = self.conv.get_user(self.user_id)
        self.timestamp = conv_event.timestamp
        self.text = conv_event.text.strip() if isinstance(conv_event, hangups.ChatMessageEvent) else ''

    def print_debug(self):
        """Print informations about conversation event"""
        print(_('Conversation ID: {}').format(self.conv_id))
        print(_('Conversation name: {}').format(get_conv_name(self.conv, truncate=True)))
        print(_('User ID: {}').format(self.user_id))
        print(_('User name: {}').format(self.user.full_name))
        print(_('Timestamp: {}').format(self.timestamp.astimezone(tz=None).strftime('%Y-%m-%d %H:%M:%S')))
        print(_('Text: {}').format(self.text))
        print()


class EventHandler:
    """Register event handlers"""
    def __init__(self):
        self.handlers = []
        self.counter = itertools.count()

    def register(self, *args, priority=10, event=None):
        """Decorator for registering event handler"""
        def wrapper(func):
            # Automatically wrap handler function in coroutine
            func = asyncio.coroutine(func)
            entry = (priority, next(self.counter), func, event)
            self.handlers.append(entry)
            self.handlers.sort()
            return func

        # If there is one (and only one) positional argument and this argument is callable,
        # assume it is the decorator (without any optional keyword arguments)
        if len(args) == 1 and callable(args[0]):
            return wrapper(args[0])
        else:
            return wrapper

    @asyncio.coroutine
    def handle(self, bot, event):
        """Handle event"""
        wrapped_event = ConversationEvent(bot, event)
        if logging.root.level == logging.DEBUG:
            wrapped_event.print_debug()

        # Don't handle event if it is produced by bot
        if wrapped_event.user.is_self:
            return

        # Run all event handlers
        for prio, i, func, event_type in self.handlers:
            if event_type is None or isinstance(event, event_type):
                try:
                    yield from func(bot, wrapped_event)
                except StopEventHandling:
                    break
                except Exception as e:
                    print(e)


# Create EventHandler singleton
handler = EventHandler()

# Build list of handlers
_plugins = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))
__all__ = [os.path.splitext(os.path.basename(f))[0] for f in _plugins
           if os.path.isfile(f) and not os.path.basename(f).startswith("_")]

# Load all handlers
from hangupsbot.handlers import *
